﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using WebSocket4Net;


/// 
/// Un simple exemple d'application
/// pour montrer comment se connecter au jeu DR2D (http://strat-num.fr/DR2D/)
/// en utilisant la librairie webSockets (WebSocket4Net.dll)
/// 
namespace WSConsoleStarter
{
	/// <summary>
	/// Interface minimum qui permet de se connecter, 
	/// de recevoir les messages et de renvoyer les réponses.
	/// </summary>
	class Program
	{
		static String _lastMessage;	// le dernier message reçu
		static String _replyMessage;// le dernier message envoyé

		static void Main(string[] args)
		{
			Console.Out.WriteLine("--------------------------------------------------------------------------------");
			Console.Out.WriteLine(" Programme de demonstration de l'implémentation d'un client/Console en C#");
			Console.Out.WriteLine(" pour répondre aux sollicitations du jeu DR2D à travers les webSockets.");
			Console.Out.WriteLine(" http://www.strat-num.fr/DR2D/\n");
			Console.Out.WriteLine("--------------------------------------------------------------------------------\n");
			
			Console.Out.Write("Entrez l'adresse du serveur de jeu sous la forme ws://{HOST}:8181/dr2d\n");
			Console.Out.Write("Par exemple ws://127.0.0.1:8181/dr2d\n\n");
			Console.Out.Write("Websocket url : ");
			var url	= Console.In.ReadLine();
			// L'adresse du serveur peut être sous forme numérique ou sous forme de nom de machine. 
			// Si une forme ne fonctionne pas, essayer l'autre.
			if(String.IsNullOrEmpty(url)) url = "ws://127.0.0.1:8181/dr2d";

			try
			{
				WebSocket socket 		= new WebSocket(url);
				socket.Opened			+= OnOpened;
				socket.Closed			+= OnClosed;
				socket.Error			+= OnError;
				socket.MessageReceived	+= OnMessageReceived;
				socket.AllowUnstrustedCertificate	= true;
				socket.Open();

				Console.Out.WriteLine("Press any key to exit");
				Console.In.ReadLine();
			}
			catch(Exception e)
			{
				Console.Out.WriteLine("Global error : " + e.Message);
			}
		}

		// ----------------------------------------------------------------------------------------
		/// <summary>
		/// Event raised when a message is received
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		static void OnMessageReceived(object sender, MessageReceivedEventArgs e)
		{
			_lastMessage = e.Message;
			Console.Out.WriteLine(" + Message received " + e.Message+"\n");
			if (_lastMessage.Contains("stgyCall"))
			{
				// reçu message de la forme:
				//{
				//	"stgyCall": { 
				//		"seq": "GameInitialisation", // un identifiant de séquence
				//		"token": "123456789", // 
				//		"argsIn": {
				//			"gameContext": null
				//		},
				//		"argsRef": null
				//	}
				//}
				// Une réponse valide, la plus élémentaire, consiste à renvoyer le contenu de l'appel 
				// en changeant "stgyCall" par "stgyReply" et en supprimant le champ argsIn.
 				//{
				//	"stgyReply": { 
				//		"seq": "GameInitialisation", // un identifiant de séquence
				//		"token": "123456789", // 
				//		"argsRef": null
				//	}
				//}

				_replyMessage = _lastMessage.Replace("stgyCall","stgyReply");
				_replyMessage = Regex.Replace(_replyMessage, "(\"argsIn\".*)\"argsRef\"","\"argsRef\"");
				
				Console.Out.WriteLine(" + Replying auomaticaly " + _replyMessage + "\n");

				((WebSocket) sender).Send(_replyMessage);
			}
		}

		// ----------------------------------------------------------------------------------------
		/// <summary>
		/// Event raised when an error is received
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		static void OnError(object sender, SuperSocket.ClientEngine.ErrorEventArgs e)
		{
			Console.Out.WriteLine(" + Socket error " + e.Exception.Message);
			//((WebSocket) sender).Close(); // not necessary as already closed.
			Console.Out.WriteLine("Press any key to exit");
		}

		// ----------------------------------------------------------------------------------------
		/// <summary>
		/// Event raised when the socket is closed
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		static void OnClosed(object sender, EventArgs e)
		{
			Console.Out.WriteLine(" + Socket closed");

		}


		// ----------------------------------------------------------------------------------------
		/// <summary>
		/// Event raised when the socket is opened
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		static void OnOpened(object sender, EventArgs e)
		{
			Console.Out.WriteLine(" + Socket opened ("+((WebSocket) sender).Version+")\n");
			((WebSocket) sender).Send(@"{ 
											""ShortMessageService"": 
											{
												""msgCode"": ""0"",
												""msgDesc"": ""Hello!"",
												""msgArgs"": null
											}
										}");
		}
	}
}
